/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.codegen;

import java.util.Iterator;
import java.util.LinkedHashSet;

import de.uni_paderborn.fujaba.metamodel.FClass;
import de.uni_paderborn.fujaba.metamodel.FPackage;


/**
 * @author    <a href="mailto:creckord@upb.de">Carsten Reckord</a>
 * @version   $Revision: 1.3 $
 */

public class ImportFragment extends TextFragment
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final transient OOGenStrategyClient client;


   /**
    * Constructor for class ImportFragment
    *
    * @param client  No description provided
    */
   public ImportFragment (OOGenStrategyClient client)
   {
      super();
      this.client = client;
   }


   /**
    * Constructor for class ImportFragment
    *
    * @param name    No description provided
    * @param client  No description provided
    */
   public ImportFragment (String name, OOGenStrategyClient client)
   {
      super (name);
      this.client = client;
   }


   /**
    * Get the client attribute of the ImportFragment object
    *
    * @return   The client value
    */
   public OOGenStrategyClient getClient()
   {
      return this.client;
   }


   /*
    *  (non-Javadoc)
    *  @see de.uni_paderborn.fujaba.codegen.CodeGenFragment#getSourceCode()
    */
   /**
    * Get the sourceCode attribute of the ImportFragment object
    *
    * @return   The sourceCode value
    */
   public String getSourceCode()
   {
      StringBuffer codeBuffer = new StringBuffer();
      if (imports != null)
      {
         Iterator iter = imports.iterator();
         while (iter.hasNext())
         {
            String importStr = (String) iter.next();
            codeBuffer.append (importStr);
         }
      }
      return codeBuffer.toString();
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private LinkedHashSet imports;


   /**
    * Access method for an one to n association.
    *
    * @param value  The object added.
    */
   public void addPackage (FPackage value)
   {
      addImport (client.generateImportPackage (value));
   }


   /**
    * Access method for an one to n association.
    *
    * @param value  The object added.
    */
   public void addClass (FClass value)
   {
      addImport (client.generateImportClass (value));
   }


   /**
    * Access method for an one to n association.
    *
    * @param importString  The object added.
    */
   protected void addImport (String importString)
   {
      if (imports == null)
      {
         imports = new LinkedHashSet();
      }
      imports.add (importString);
   }
}

/*
 * $Log: ImportFragment.java,v $
 * Revision 1.3  2004/10/20 17:49:37  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
