/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.coobra.actions;

import java.awt.event.ActionEvent;

import javax.swing.*;

import de.tu_bs.coobra.LocalRepository;
import de.tu_bs.coobra.RemoteRepositoryManager;
import de.tu_bs.coobra.remote.RemoteRepository;
import de.tu_bs.coobra.remote.lightweight.*;
import de.tu_bs.coobra.util.Password;
import de.uni_paderborn.fujaba.app.FrameMain;
import de.uni_paderborn.fujaba.coobra.FujabaChangeManager;
import de.uni_paderborn.fujaba.uml.UMLProject;


/**
 * Restore all changes from server
 *
 * @author    $Author: cschneid $
 * @version   $Revision: 1.25 $
 */
public class CheckoutAction extends AbstractAction implements LocalRepository.ProgressCallback
{
   /**
    * For choosing the server to checkout from
    */
   SelectServerRepositoryDialog dialog;
   /**
    * true if no update should be called
    */
   private boolean connectOnly;

   /**
    * dialog for displaying progress
    */
   ProgressDialog progress = new ProgressDialog (FrameMain.get());


   /**
    * called when update/restore/store progress
    *
    * @param percent  progress percentage
    */
   public void progress (int percent)
   {
      if (progress != null)
      {
         //progress.setVisible( percent > 0 && percent < 100 );
         progress.setPercentage (percent);
      }
   }


   /**
    * Perform the action
    *
    * @param event  the ActionEvent for performing the action
    */
   public void actionPerformed (ActionEvent event)
   {
      String status;
      connectOnly = FujabaChangeManager.getVMRepository() != null && FujabaChangeManager.getVMRepository().getServerIdentifier() != null;

      //ask where to checkout from
      if (dialog == null)
      {
         dialog = new SelectServerRepositoryDialog (FrameMain.get(), true);
      }

      if (connectOnly)
      {
         dialog.setTitle ("Connect");
         progress.setTitle ("Connect");
      }
      else
      {
         dialog.setTitle ("Checkout");
         progress.setTitle ("Checkout");
      }
      dialog.show();

      progress.setPercentage (0);

      if (dialog.isOk())
      {
         connectOnly |= dialog.isConnectOnlyChecked();
         status = "connecting";
         FrameMain.get().setStatusLabel (status);

            new Thread()
            {
               /**
                * @see   Runnable#run()
                */
               public void run()
               {
                  while (!progress.isVisible())
                  {
                     try
                     {
                        Thread.sleep (10);
                     }
                     catch (InterruptedException e)
                     {
                     }
                  }
                  try
                  {
                     connect();
                  }
                  finally
                  {
                     progress.hide();
                  }
               }

            }.start();
         progress.show();
      }
   }


   /**
    * perform connect (and update)
    */
   void connect()
   {
      LocalRepository oldRepo = FujabaChangeManager.getVMRepository();
      String status;
      try
      {
         LocalRepository repository;
         if (!connectOnly)
         {
            UMLProject.setNewProject (null);
            repository = createRepository();

            FujabaChangeManager.setVMRepository (repository);
            if (oldRepo != null)
            {
               oldRepo.removeYou();
            }
         }
         else
         {
            repository = oldRepo;
         }

         RemoteRepository remote;
         if (dialog.isByName())
         {
            if (!dialog.isLightweightChecked())
            {
               RemoteRepositoryManager manager = new RemoteRepositoryManager (dialog.getNameServer(), dialog.getNameServerPort(), null);
               remote = manager.connectByName (dialog.getRepositoryName());
            }
            else
            {
               if (dialog.isNameServerChecked())
               {
                  LightWeightNameService.ServiceInfo info =
                     LightWeightNameService.query (dialog.getNameServer(),
                     Integer.parseInt (dialog.getNameServerPort()),
                     dialog.getRepositoryName());
                  final String passwordHash = Password.passwordHash (dialog.getPassword());
                  remote = new RemoteRepository (new LightWeightServer (info.getHost(),
                     Integer.parseInt (info.getPort()),
                     dialog.getLogin(),
                     passwordHash));
               }
               else
               {
                  final String passwordHash = Password.passwordHash (dialog.getPassword());
                  remote = new RemoteRepository (new LightWeightServer (dialog.getNameServer(),
                     Integer.parseInt (dialog.getNameServerPort()),
                     dialog.getLogin(),
                     passwordHash));
               }
            }
         }
         else
         {
            RemoteRepositoryManager manager = new RemoteRepositoryManager (null);
            remote = manager.connectByIOR (dialog.getIOR());
         }

         if (connectOnly && repository.getServerIdentifier() != null
            && !remote.getServerIdentifier().equals (repository.getServerIdentifier()))
         {
            JOptionPane.showMessageDialog (FrameMain.get(), "The server does not contain the Project you are working on. Connect aborted!"
               , "Connect", JOptionPane.ERROR_MESSAGE);
            return;
         }

         repository.connect (remote);
         status = "connect succeeded";

         dialog.rememberCurrentDataInHistoryList();

         if (!connectOnly)
         {
            repository.update (this);

            //find project
            UMLProject project = FujabaChangeManager.getUMLProject();
            if (project != null)
            {
               UMLProject.setNewProject (project);
               FrameMain.get().setUMLProject (project);
               project.refreshDisplay();
               repository.moveChangesIntoChangesBase();
               project.setSaved (true);
            }
            else
            {
               status = "No project was found in the loaded repository!";
               //repository.removeYou();
            }
         }

         FrameMain.get().setStatusLabel (status);
      }
      catch (Exception e)
      {
         FrameMain.get().setStatusLabel ("connect failed: " + e.getMessage());
         if (e.getMessage() != null)
         {
            if (e.getMessage().indexOf (LightWeightServerImplWithAuthentication.ERROR_AUTH) >= 0)
            {
               JOptionPane.showMessageDialog (FrameMain.get(), "Username and password not accepted!"
                  , "Connect", JOptionPane.ERROR_MESSAGE);
            }
            else if (e.getMessage().indexOf (LightWeightServerImplWithAuthentication.ERROR_DENIED_PREFIX) >= 0)
            {
               JOptionPane.showMessageDialog (FrameMain.get(), "User " + dialog.getLogin() + " is not allowed to connect to repository " + dialog.getRepositoryName()
                  , "Connect", JOptionPane.ERROR_MESSAGE);
            }
         }
         e.printStackTrace();
      }
   }


   /**
    * @return   new LocalRepository()
    */
   protected LocalRepository createRepository()
   {
      return new LocalRepository();
   }
}

/*
 * $Log: CheckoutAction.java,v $
 * Revision 1.25  2004/12/09 10:06:02  cschneid
 * connect allows connections without name server now, libraries updated
 *
 */
