/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa;

import java.awt.*;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.border.BevelBorder;

import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.fsa.actions.AddBendAction;
import de.uni_paderborn.fujaba.fsa.swing.*;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.metamodel.FElement;
import de.uni_paderborn.fujaba.uml.UMLAssoc;


/**
 * Class FSAPolyLine
 *
 * @author    $Author: fklar $
 * @version   $Revision: 1.25 $
 */
public class FSAPolyLine
    extends FSABendLine
{

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public final static String BEND_LOCATION = "bendLocation_";


   /**
    * Constructor for class FSAPolyLine
    *
    * @param incr      No description provided
    * @param propName  No description provided
    */
   public FSAPolyLine (LogicUnparseInterface incr, String propName)
   {
      super (incr, propName);
   }


   /**
    * Constructor for class FSAObject
    *
    * @param incr          No description provided
    * @param propName      No description provided
    * @param parent        No description provided
    * @param myJComponent  No description provided
    */
   public FSAPolyLine (LogicUnparseInterface incr,
                       String propName,
                       JComponent parent,
                       JComponent myJComponent)
   {
      super (incr, propName, parent, myJComponent);
   }


   /**
    * Constructor for class FSAPolyLine
    *
    * @param incr      No description provided
    * @param propName  No description provided
    * @param parent    No description provided
    */
   public FSAPolyLine (LogicUnparseInterface incr, String propName, JComponent parent)
   {
      super (incr, propName, parent);
      if (incr instanceof UMLAssoc && "items".equals ( ((UMLAssoc) incr).getName()))
      {
         getJComponent().setBorder (new BevelBorder (BevelBorder.RAISED));
      }
   }


   /**
    * create empty JPolyLine
    *
    * @return   No description provided
    */
   protected JComponent createJComponent()
   {
      JPolyLine line = new JPolyLine (new JBend (new Point (0, 0)), new JBend (new Point (0, 0)));
      line.setBackground (FSAObject.COLOR_BACKGROUND);
      line.setForeground (FSAObject.COLOR_FOREGROUND);
      return  (line);
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void arrangeLoopBends()
   {
      JPolyLine polyLine = (JPolyLine) getJComponent();

      FSABend start = getStartBend();
      FSABend end = getEndBend();
      if ( (start instanceof FSAGrab) &&  (end instanceof FSAGrab))
      {
         FSAObject startObj =  ((FSAGrab) start).getTarget();
         FSAObject endObj =  ((FSAGrab) end).getTarget();
         if ( (startObj != null) &&  (startObj.equals (endObj)))
         {
            Point point = new Point (startObj.getLocation());
            point.translate (-30, 10);
            FSABend bend = AddBendAction.newBend (this, point);
            polyLine.insertInBends (1, (JBend) bend.getJComponent());
            point.translate (0, -40);
            bend = AddBendAction.newBend (this, point);
            polyLine.insertInBends (2, (JBend) bend.getJComponent());
            point.translate (40, 0);
            bend = AddBendAction.newBend (this, point);
            polyLine.insertInBends (3, (JBend) bend.getJComponent());
         }
      }
   }


   /**
    * Sets the startBend attribute of the FSAPolyLine object
    *
    * @param bend  The new startBend value
    */
   public void setStartBend (FSABend bend)
   {
      super.setStartBend (bend);
      if ( ((JPolyLine) getJComponent()).sizeOfBends() == 2)
      {
         arrangeLoopBends();
      }
   }


   /**
    * Sets the endBend attribute of the FSAPolyLine object
    *
    * @param bend  The new endBend value
    */
   public void setEndBend (FSABend bend)
   {
      super.setEndBend (bend);
      if ( ((JPolyLine) getJComponent()).sizeOfBends() == 2)
      {
         arrangeLoopBends();
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   public void removeYou()
   {
      JBendLine line = (JBendLine) getJComponent();
      line.removeYou();
      super.removeYou();
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param force  No description provided
    */
   public void removeYouWithBends (boolean force)
   {
      JBendLine line = (JBendLine) getJComponent();
      if (line != null)
      {
         LinkedList bends = null;
         Iterator iter = line.iteratorOfBends();
         while (iter.hasNext())
         {
            JBend bend = (JBend) iter.next();

            if (bend != null)
            {
               if (bends == null)
               {
                  bends = new LinkedList();
               }
               bends.add (bend);
            }
         }
         removeYou();
         if (bends != null)
         {
            iter = bends.iterator();
            while (iter.hasNext())
            {
               JBend bend = (JBend) iter.next();
               if (force || bend.sizeOfLines() == 0)
               {
                  FSAObject fsa = FSAObject.getFSAObjectFromJComponent (bend);
                  if (fsa != null)
                  {
                     fsa.removeYou();
                  }
                  bend.removeAllFromLines();
                  Container parent = bend.getParent();
                  if (parent != null)
                  {
                     parent.remove (bend);
                  }
               }
            }
         }
      }
      else
      {
         removeYou();
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected void oldInitLocation()
   {
      LogicUnparseInterface incr = getLogic();
      if (incr instanceof ASGElement)
      {
         ASGElement asgElement = (ASGElement) incr;
         int i = 1;
         FElement asgParent = getFSAQualifier();
         //getParent() != null && getParent().getLogic() instanceof ASGElement ? (ASGElement) getParent().getLogic() : null;
         Point point = asgElement.getPointFromUnparseInformation ((ASGElement) asgParent, "bend" + i);
         //for loading old fprs:
         if (point == null)
         {
            point = asgElement.getPointFromUnparseInformation (asgElement, "bend" + i);
         }

         while (point != null)
         {
            FSABend bend = AddBendAction.newBend (this, point);
             ((JPolyLine) getJComponent()).insertInBends (i, (JBend) bend.getJComponent());

            i++;
            point = asgElement.getPointFromUnparseInformation ((ASGElement) asgParent, "bend" + i);
            //for loading old fprs:
            if (point == null)
            {
               point = asgElement.getPointFromUnparseInformation (asgElement, "bend" + i);
            }
         }
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected void initFSAProperties()
   {
      super.initFSAProperties();

      int i = 1;
      Point bendLocation = getPointFromUnparseInformation (BEND_LOCATION + i);
      while (bendLocation != null)
      {
         FSABend bend = AddBendAction.newBend (this, bendLocation);
          ((JPolyLine) getJComponent()).insertInBends (i, (JBend) bend.getJComponent());

         i++;
         bendLocation = getPointFromUnparseInformation (BEND_LOCATION + i);
      }

      //try to load old fpr bend information
      if (i == 1)
      {
         oldInitLocation();
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected void saveLocation()
   {
      super.saveLocation();

      JPolyLine polyLine = (JPolyLine) getJComponent();
      int numberOfBends = polyLine.sizeOfBends();
      for (int i = 1; i < numberOfBends - 1; i++)
      {
         JBend bend = polyLine.getFromBends (i);
         addPointToUnparseInformation (BEND_LOCATION + i, bend.getLocation());
      }
   }
}

/*
 * $Log: FSAPolyLine.java,v $
 * Revision 1.25  2005/01/25 18:10:58  fklar
 * more than one additional bend in a FSAPolyLine is now saved
 *
 */
