/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.swing;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.*;


/**
 * show with JLabel , edit with JComboBox
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.16 $
 */
public abstract class AbstractJComboBoxLabel
    extends EditableTextComponent implements ItemListener
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private ComboBoxModel model = null;


   /**
    * Sets the model attribute of the AbstractJComboBoxLabel object
    *
    * @param model  The new model value
    */
   public void setModel (ComboBoxModel model)
   {
      this.model = model;
   }


   /**
    * Get the model attribute of the AbstractJComboBoxLabel object
    *
    * @return   The model value
    */
   public ComboBoxModel getModel()
   {
      return this.model;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private ListCellRenderer renderer = null;


   /**
    * Sets the renderer attribute of the AbstractJComboBoxLabel object
    *
    * @param renderer  The new renderer value
    */
   public void setRenderer (ListCellRenderer renderer)
   {
      this.renderer = renderer;
   }


   /**
    * Get the renderer attribute of the AbstractJComboBoxLabel object
    *
    * @return   The renderer value
    */
   public ListCellRenderer getRenderer()
   {
      return this.renderer;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param component  No description provided
    */
   protected void updateEditor (JComponent component)
   {
      if (component instanceof JComboBox)
      {
         JComboBox comboBox = (JComboBox) component;
         // notify JComponent's
         if (comboBox.getSelectedItem() != getValue())
         {
            updateJComboxBox (comboBox);
         } // end of if ()
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param component  No description provided
    */
   protected void enableEditor (JComponent component)
   {
      if (component instanceof JComboBox)
      {
         super.enableEditor (component);

         JComboBox comboBox = (JComboBox) component;

         comboBox.setModel (getModel());
         comboBox.setRenderer (getRenderer());

         comboBox.setBackground (getBackground());
         comboBox.setForeground (getForeground());
         comboBox.setFont (getFont());
         comboBox.updateUI();

         updateJComboxBox (comboBox);

         comboBox.addItemListener (this);
      } // end of if ()
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param component  No description provided
    */
   protected void disableEditor (JComponent component)
   {
      if (component instanceof JComboBox)
      {
         super.disableEditor (component);

         JComboBox comboBox = (JComboBox) component;
         comboBox.removeItemListener (this);
      }
   }


   /**
    * Get the editorClassName attribute of the AbstractJComboBoxLabel object
    *
    * @return   The editorClassName value
    */
   protected String getEditorClassName()
   {
      return JComboBox.class.getName();
   }


   /**
    * Overwrites the itemStateChanged method of ItemListener.
    *
    * @param event  No description provided
    */
   public void itemStateChanged (ItemEvent event)
   {
      Object component = event.getSource();
      if ( (component instanceof JComboBox) &&
         event.getStateChange() == ItemEvent.SELECTED)
      {
         JComboBox comboBox = (JComboBox) component;
         setValue (comboBox.getSelectedItem());

         if (isEditing())
         {
            setDisplayMode();
         }
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param comboBox  No description provided
    */
   protected abstract void updateJComboxBox (JComboBox comboBox);


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   protected abstract void updateJLabel();
}

/*
 * $Log: AbstractJComboBoxLabel.java,v $
 * Revision 1.16  2004/11/03 10:17:56  lowende
 * Javadoc warnings removed.
 *
 */
