/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.uml.unparse;

import java.awt.*;

import javax.swing.*;

import de.uni_paderborn.fujaba.fsa.*;
import de.uni_paderborn.fujaba.fsa.listener.*;
import de.uni_paderborn.fujaba.fsa.swing.*;
import de.uni_paderborn.fujaba.fsa.swing.border.CommentBorder;
import de.uni_paderborn.fujaba.fsa.unparse.AbstractUnparseModule;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.fsa.update.*;
import de.uni_paderborn.fujaba.uml.UMLClass;
import de.uni_paderborn.fujaba.uml.UMLCommentary;


/**
 * No comment provided by developer, please add a comment to improve documentation.
 *
 * @author    $Author: trinet $
 * @version   $Revision: 1.10 $
 */
public class UMCommentary extends AbstractUnparseModule
{

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param parent  No description provided
    * @param incr    No description provided
    * @return        No description provided
    */
   public FSAObject create (FSAObject parent, LogicUnparseInterface incr)
   {
      UMLCommentary comment = (UMLCommentary) incr;
      //this seems to work for class comments only
      if (comment.getRevComment() instanceof UMLClass)
      {

         FSASeparatedPanel commentPanel = new FSASeparatedPanel (incr, getMainFsaName(), parent.getJComponent());
         commentPanel.setTransientProperties (false);
         commentPanel.setBorder (new CommentBorder (Color.black));

         JComponent commentPanelJComponent = commentPanel.getJComponent();
         commentPanelJComponent.addKeyListener (ArrowMoveListener.get());
         //AscendDescendMouseHandler.addMouseInputListener (commentPanelJComponent, ComponentCursorListener.get());
         AscendDescendMouseHandler.addMouseInputListener (commentPanelJComponent, SelectionMouseListener.get());
         AscendDescendMouseHandler.addMouseInputListener (commentPanelJComponent, DragMouseListener.get());
         AscendDescendMouseHandler.addMouseInputListener (commentPanelJComponent, DoubleClickMouseListener.get());
         SelectionListenerHelper.addSelectionListener (commentPanelJComponent, BorderHighlighter.get());
         SelectionListenerHelper.addSelectionListener (commentPanelJComponent, ArrowMoveListener.get());

         // the name of the object.
         FSATextComponent nameLabel = new FSATextComponent (incr, "text", commentPanel.getJComponent());
         nameLabel.addToUpdater (nameLabel.createDefaultUpdater());

         FSABendLine line = new FSABendLine (incr, getMainFsaName() + ".line", parent.getJComponent());
         JComponent jPanel = line.getJComponent();
          ((JLine) jPanel).setUI (DashedLineUI.createUI (jPanel));

         // add mouse listener for selecting
         AscendDescendMouseHandler.addMouseInputListener (jPanel, SelectionMouseListener.get());
         SelectionListenerHelper.addSelectionListener (jPanel, ForegroundHighlighter.get());

         FSAGrab grab = new FSAGrab (incr, "startGrab", parent.getJComponent());
         PanelGrabUI ui = (PanelGrabUI) PanelGrabUI.createUI (grab.getJComponent());
         grab.setUI (ui);
         grab.setTarget (commentPanel);
         grab.getJComponent().setOpaque (false);
         line.setStartBend (grab);

         grab = new FSAEntryGrab (incr, "exitGrab", parent.getJComponent());
         LogicToFsaUpdater updater = new LogicToFsaUpdater();
         Translator translator = new LogicFsaTranslator (line, 1);
         updater.setTranslator (translator);
         updater.setLogicAttrName ("revComment");
         updater.setFsaAttrName ("target");
         updater.setLogicObject (comment);
         grab.addToUpdater (updater);
         line.setEndBend (grab);
         JGrab jgrab = (JGrab) grab.getJComponent();
         SelectionListenerHelper.addSelectionListener (jgrab, ForegroundHighlighter.get());
         AscendDescendMouseHandler.addMouseInputListener (jgrab, GrabMouseListener.get());

         return commentPanel;
      }
      else
      {
         return null;
      }
   }
}

/*
 * $Log: UMCommentary.java,v $
 * Revision 1.10  2004/11/20 15:55:07  trinet
 * added flag to avoid unnecessary saving of unparse information
 *
 */
