/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import java.util.Iterator;

import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.uml.UMLDiagramItem;
import de.upb.tools.fca.FEmptyIterator;
import de.upb.tools.fca.FPropHashSet;


/**
 * A Use Case of a {@link UsecaseDiagram}.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.21 $
 * @see       UsecaseDiagram
 */
public class Usecase extends UMLDiagramItem
{
   /**
    * Constructor for class Usecase
    */
   public Usecase()
   {
      setName ("Use Case " + getID());
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public String toString()
   {
      return getName();
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String name = "";


   /**
    * Get the name attribute of the Usecase object
    *
    * @return   The name value
    */
   public String getName()
   {
      return name;
   } // getName


   /**
    * Set the value of name.
    *
    * @param name  Value to assign to name.
    */
   public void setName (String name)
   {
      if ( (this.name == null && name != null) ||
          (this.name != null && !this.name.equals (name)))
      {
         String oldValue = this.name;

         this.name = name;

         firePropertyChange ("name", oldValue, this.name);
      }
   }



   /**
    * <pre>
    *               n     usages      1
    * UsecaseUsage --------------------- Usecase
    *               usages      usecase
    * </pre>
    */

   private FPropHashSet usages;


   /**
    * UMLMethod: '+ hasInUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean hasInUsages (UsecaseUsage value)
   {
      return  ( (this.usages != null) &&
          (value != null) &&
         this.usages.contains (value));
   }


   /**
    * UMLMethod: '+ iteratorOfUsages () : Iterator'
    *
    * @return   No description provided
    */
   public Iterator iteratorOfUsages()
   {
      return  ( (this.usages == null)
         ? FEmptyIterator.get()
         : this.usages.iterator());
   }


   /**
    * UMLMethod: '+ sizeOfUsages () : Integer'
    *
    * @return   No description provided
    */
   public int sizeOfUsages()
   {
      return  ( (this.usages == null)
         ? 0
         : this.usages.size());
   }


   /**
    * UMLMethod: '+ addToUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  The object added.
    * @return       No description provided
    */
   public boolean addToUsages (UsecaseUsage value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.usages == null)
         {
            this.usages = new FPropHashSet (this, "usages"); // or FTreeSet () or FLinkedList ()

         }
         changed = this.usages.add (value);
         if (changed)
         {
            value.setUsecase (this);
         }
      }
      return changed;
   }


   /**
    * UMLMethod: '+ removeFromUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean removeFromUsages (UsecaseUsage value)
   {
      boolean changed = false;
      if ( (this.usages != null) &&  (value != null))
      {
         changed = this.usages.remove (value);
         if (changed)
         {
            value.setUsecase (null);
         }
      }
      return changed;
   }


   /**
    * UMLMethod: '+ removeAllFromUsages () : Void'
    */
   public void removeAllFromUsages()
   {
      UsecaseUsage tmpValue;
      Iterator iter = this.iteratorOfUsages();
      while (iter.hasNext())
      {
         tmpValue = (UsecaseUsage) iter.next();
         this.removeFromUsages (tmpValue);
      }
   }



   /**
    * <pre>
    *                 0..1          usecases           n
    * UsecaseDiagram ------------------------------------ Usecase
    *                 parentUsecaseDiagram      usecases
    * </pre>
    */
   private UsecaseDiagram parentUsecaseDiagram;


   /**
    * Sets the parentUsecaseDiagram attribute of the Usecase object
    *
    * @param value  The new parentUsecaseDiagram value
    * @return       No description provided
    */
   public boolean setParentUsecaseDiagram (UsecaseDiagram value)
   {
      boolean changed = false;
      if (this.parentUsecaseDiagram != value)
      {
         if (this.parentUsecaseDiagram != null)
         {
            UsecaseDiagram oldValue = this.parentUsecaseDiagram;
            this.parentUsecaseDiagram = null;
            oldValue.removeFromUsecases (this);
         }
         this.parentUsecaseDiagram = value;
         if (value != null)
         {
            value.addToUsecases (this);
         }
         changed = true;
      }
      return changed;
   }


   /**
    * Get the parentUsecaseDiagram attribute of the Usecase object
    *
    * @return   The parentUsecaseDiagram value
    */
   public UsecaseDiagram getParentUsecaseDiagram()
   {
      return this.parentUsecaseDiagram;
   }


   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      Iterator iter = iteratorOfUsages();
      while (iter.hasNext())
      {
         UsecaseUsage usecaseUsage = (UsecaseUsage) iter.next();
         usecaseUsage.removeYou();
      }
      removeAllFromUsages();

      this.setParentUsecaseDiagram (null);
      super.removeYou();
   }


   /**
    * Searches the ASG tree for a given id
    *
    * @param id  The id to search for.
    * @return    The Element with the given id, null if not found.
    */
   public ASGElement searchID (String id)
   {
      ASGElement elem = super.searchID (id);
      Iterator iter = iteratorOfUsages();
      while ( (elem == null) &&  (iter.hasNext()))
      {
         elem =  ((UsecaseUsage) iter.next()).searchID (id);
      }
      return elem;
   }
}

/*
 * $Log: Usecase.java,v $
 * Revision 1.21  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
