/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import de.uni_paderborn.fujaba.uml.UMLDiagramItem;


/**
 * The usage of a {@link Usecase} by a {@link UsecaseActor}.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.13 $
 */
public class UsecaseUsage extends UMLDiagramItem
{
   /**
    * <pre>
    *          1     usages      n
    * Usecase --------------------- UsecaseUsage
    *          usecase      usages
    * </pre>
    */

   private Usecase usecase;


   /**
    * UMLMethod: '+ setUsecase (value : Usecase) : Boolean'
    *
    * @param value  The new usecase value
    * @return       No description provided
    */
   public boolean setUsecase (Usecase value)
   {
      boolean changed = false;
      if (this.usecase != value)
      {
         if (this.usecase != null)
         {
            Usecase oldValue = this.usecase;
            this.usecase = null;
            oldValue.removeFromUsages (this);
         }
         this.usecase = value;
         if (value != null)
         {
            value.addToUsages (this);
         }
         changed = true;
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getUsecase () : Usecase'
    *
    * @return   The usecase value
    */
   public Usecase getUsecase()
   {
      return this.usecase;
   }



   /**
    * <pre>
    *               1    usages     n
    * UsecaseActor ------------------- UsecaseUsage
    *               actor      usages
    * </pre>
    */

   private UsecaseActor actor;


   /**
    * UMLMethod: '+ setActor (value : UsecaseActor) : Boolean'
    *
    * @param value  The new actor value
    * @return       No description provided
    */
   public boolean setActor (UsecaseActor value)
   {
      boolean changed = false;
      UsecaseActor oldValue = this.actor;
      if (this.actor != value)
      {
         if (this.actor != null)
         {
            this.actor = null;
            oldValue.removeFromUsages (this);
         }
         this.actor = value;
         firePropertyChange ("actor", oldValue, value);
         if (value != null)
         {
            value.addToUsages (this);
         }
         changed = true;
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getActor () : UsecaseActor'
    *
    * @return   The actor value
    */
   public UsecaseActor getActor()
   {
      return this.actor;
   }



   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      Usecase tmpUsecase = getUsecase();
      if (tmpUsecase != null)
      {
         setUsecase (null);
      } // if

      UsecaseActor tmpActor = getActor();
      if (tmpActor != null)
      {
         setActor (null);
      } // if
      super.removeYou();
   }
}

/*
 * $Log: UsecaseUsage.java,v $
 * Revision 1.13  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
