/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.unparse;

import javax.swing.*;

import de.uni_paderborn.fujaba.fsa.FSALayeredPane;
import de.uni_paderborn.fujaba.fsa.FSAObject;
import de.uni_paderborn.fujaba.fsa.listener.*;
import de.uni_paderborn.fujaba.fsa.swing.LayerManager;
import de.uni_paderborn.fujaba.fsa.swing.NoLayout;
import de.uni_paderborn.fujaba.fsa.unparse.FSAInterface;
import de.uni_paderborn.fujaba.fsa.unparse.LogicUnparseInterface;
import de.uni_paderborn.fujaba.uml.unparse.UMDiagram;


/**
 * Class for displaying a {@link de.uni_paderborn.fujaba.usecase.UsecaseSystemDiagram} on screen.
 * A UsecaseSystemDiagram is used to implement the rule, that all {@link de.uni_paderborn.fujaba.usecase.Usecase}s
 * have to be inside the System-box, see {@link de.uni_paderborn.fujaba.usecase.UsecaseSystem}.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.20 $
 * @see       de.uni_paderborn.fujaba.usecase.Usecase
 * @see       de.uni_paderborn.fujaba.usecase.UsecaseSystem
 * @see       de.uni_paderborn.fujaba.usecase.UsecaseSystemDiagram
 */
public class UMUsecaseSystemDiagram extends UMDiagram
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int LAYER = JLayeredPane.DRAG_LAYER.intValue() + 10;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static int DRAG_LAYER = LAYER + 1;
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private final static String LAYER_NAME = "UseCaseSystemDiagramLayer";

   static
   {
      LayerManager.get().addToLayerPositions (LAYER_NAME, new Integer (LAYER));
      LayerManager.get().addToLayerPositions (LayerManager.DRAG_LAYER_PREFIX + LAYER_NAME, new Integer (DRAG_LAYER));
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param parent  No description provided
    * @param incr    No description provided
    * @return        No description provided
    */
   public FSAObject create (FSAObject parent, LogicUnparseInterface incr)
   {
      FSALayeredPane systemDiagramPane = new FSALayeredPane (incr, "entry", FSAInterface.getJComponent (parent));

      LayerManager.setLayerType (systemDiagramPane.getJComponent(), LAYER_NAME);

      systemDiagramPane.setLayout (new NoLayout (false));
      systemDiagramPane.setOpaque (false);

      JComponent pane = systemDiagramPane.getJComponent();
      AscendDescendMouseHandler.addMouseListener (pane, PopupListener.get());
      AscendDescendMouseHandler.addMouseListener (pane, HideEditorsMouseListener.get());
      AscendDescendMouseHandler.addMouseInputListener (pane, RectangularSelectionListener.get());
      AscendDescendMouseHandler.addMouseInputListener (pane, AscendDescendMouseHandler.getDescendConsumer());

      return systemDiagramPane;
   }

}

/*
 * $Log: UMUsecaseSystemDiagram.java,v $
 * Revision 1.20  2004/10/20 17:50:29  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
